import { Request, Response, NextFunction } from "express";
import prisma from "../../database/dbclient";
import { z } from "zod";
import multer from "multer";
import path from "path";
import fs from "fs";

// Create a company document
const orgranogramSchema = z.object({
  data: z.string(),
  name: z.string().optional().nullable(),
  company_id: z.number(),
});

export const createorgranogram = async (req: Request, res: Response) => {
  const { data, error } = orgranogramSchema.safeParse(req.body);
  if (error) {
    console.error(
      "POST > Create company document > Invalid request body >",
      error
    );
    return res.status(400).json({ message: "Invalid Request body", error });
  }
  try {
    // 1st need to save the data

    if (data.data.length == 0) {
      return res.status(400).json({ message: "Data is Empty", error });
    }

    const uploadPath = path.join("/data", "uploads");
    const uniqueSuffix = Date.now() + "-" + Math.round(Math.random() * 1e9);

    const filePath = path.join(uploadPath, uniqueSuffix + ".json");

    fs.writeFileSync(filePath, data.data, "utf-8");

    const fileUrl = `https://${req.get("host")}/uploads/${path.basename(filePath)}`;

    const newDoc = await prisma.orgranogram_db.create({
      data: {
        name: data?.name ?? uniqueSuffix,
        file_url: fileUrl,
        company_id: data.company_id,
      },
    });

    return res
      .status(201)
      .json({ message: "Document created successfully", data: newDoc });
  } catch (err) {
    console.error(
      "POST > Create Company Document > Failed to create company document > Error >",
      err
    );
    return res.status(500).json({
      message:
        "Failed to create company document. View server logs for details.",
      error: err,
    });
  }
};

export const updateChart = async (req: Request, res: Response) => {
  const { chart_id } = req.params;
  const chartId = Number.parseInt(chart_id ?? "0");

  const { data, error } = orgranogramSchema.safeParse(req.body);
  if (error) {
    console.error("PATCH > Update organogram > Invalid request body >", error);
    return res.status(400).json({ message: "Invalid Request body", error });
  }
  try {
    // Find the existing record
    const chart = await prisma.orgranogram_db.findUnique({
      where: {
        id: chartId,
      },
    });

    if (!chart) {
      console.error("PATCH > Failed to get chart data >", error);
      return res.status(404).json({ message: "Data not found" });
    }

    // Check if there are graph data available
    if (data.data.length > 0) {
      // If so, update the file
      // Find the file
      const filepath = path.join(
        "/data",
        "uploads",
        chart.file_url.split("/uploads")[1]
      );
      // Overwrite the file
      fs.writeFileSync(filepath, data.data, "utf-8");
    }

    const updatedRec = await prisma.orgranogram_db.update({
      where: {
        id: chartId,
      },
      data: {
        name: data.name,
        updated_at: new Date(),
      },
    });

    return res
      .status(200)
      .json({ message: "Document updated successfully", data: updateChart });
  } catch (err) {
    console.error(
      "PATCH > Update Company organogram > Failed to update company organogram > Error >",
      err
    );
    return res.status(500).json({
      message:
        "Failed to update company organogram. View server logs for details.",
      error: err,
    });
  }
};

const chartVersionSchema = z.object({
  name: z.string(),
});

export const updateChartName = async (req: Request, res: Response) => {
  const { chart_id } = req.params;
  const chartId = Number.parseInt(chart_id ?? "0");

  const { data, error } = chartVersionSchema.safeParse(req.body);
  if (error) {
    console.error("PATCH > Update organogram > Invalid request body >", error);
    return res.status(400).json({ message: "Invalid Request body", error });
  }
  try {
    // Find the existing record
    const chart = await prisma.orgranogram_db.findUnique({
      where: {
        id: chartId,
      },
    });

    if (!chart) {
      console.error("PATCH > Failed to get chart data >", error);
      return res.status(404).json({ message: "Data not found" });
    }

    // Check if there are graph data available
    const updatedRec = await prisma.orgranogram_db.update({
      where: {
        id: chartId,
      },
      data: {
        name: data.name,
        updated_at: new Date(),
      },
    });

    return res
      .status(200)
      .json({ message: "Document updated successfully", data: updateChart });
  } catch (err) {
    console.error(
      "PATCH > Update Company organogram > Failed to update company organogram > Error >",
      err
    );
    return res.status(500).json({
      message:
        "Failed to update company organogram. View server logs for details.",
      error: err,
    });
  }
};

export const getAllOrganograms = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;

  const companyId = Number.parseInt(company_id ?? "0");

  try {
    const charts = await prisma.orgranogram_db.findMany({
      where: {
        company_id: companyId,
      },
      orderBy: {
        updated_at: "desc",
      },
    });

    return res.status(200).json(charts);
  } catch (error) {
    console.error("Failed to get all organograms >", error);
    next(error);
  }
};
